@@ -92,6 +92,10 @@
  * with special properties like no file modes */
 #undef HAVE_DOSISH_SYSTEM
 
+#undef USE_CAPABILITIES
+
+/* Some systems have mkdir that takes a single argument. */
+#undef MKDIR_TAKES_ONE_ARG
 
 
 @BOTTOM@
