@@ -39,6 +39,16 @@
 #include "trustdb.h"
 #include "hkp.h"
 
+
+struct kidlist_item {
+    struct kidlist_item *next;
+    u32 kid[2];
+    int pubkey_algo;
+    int reason;
+};
+
+
+
 /****************
  * Structure to hold the context
  */
@@ -60,6 +70,8 @@ struct mainproc_context {
     IOBUF iobuf;    /* used to get the filename etc. */
     int trustletter; /* temp usage in list_node */
     ulong local_id;    /* ditto */
+    struct kidlist_item *failed_pkenc;	/* list of packets for which
+					   we do not have a secret key */
 };
 
 
@@ -76,6 +88,12 @@ release_list( CTX c )
 	return;
     proc_tree(c, c->list );
     release_kbnode( c->list );
+    while( c->failed_pkenc ) {
+	struct kidlist_item *tmp = c->failed_pkenc->next;
+	m_free( c->failed_pkenc );
+	c->failed_pkenc = tmp;
+    }
+    c->failed_pkenc = NULL;
     c->list = NULL;
 }
 
@@ -106,7 +124,7 @@ static int
 add_user_id( CTX c, PACKET *pkt )
 {
     if( !c->list ) {
-	log_error("orphaned user id\n" );
+	log_error("orphaned user ID\n" );
 	return 0;
     }
     add_kbnode( c->list, new_kbnode( pkt ) );
@@ -124,6 +142,17 @@ add_subkey( CTX c, PACKET *pkt )
     return 1;
 }
 
+static int
+add_ring_trust( CTX c, PACKET *pkt )
+{
+    if( !c->list ) {
+	log_error("ring trust w/o key\n" );
+	return 0;
+    }
+    add_kbnode( c->list, new_kbnode( pkt ) );
+    return 1;
+}
+
 
 static int
 add_signature( CTX c, PACKET *pkt )
@@ -163,7 +192,7 @@ proc_symkey_enc( CTX c, PACKET *pkt )
 	log_error( "symkey_enc packet with session keys are not supported!\n");
     else {
 	c->last_was_session_key = 2;
-	c->dek = passphrase_to_dek( NULL, enc->cipher_algo, &enc->s2k, 0 );
+	c->dek = passphrase_to_dek( NULL, 0, enc->cipher_algo, &enc->s2k, 0 );
     }
     free_packet(pkt);
 }
@@ -185,7 +214,8 @@ proc_pubkey_enc( CTX c, PACKET *pkt )
 
     if( is_status_enabled() ) {
 	char buf[50];
-	sprintf(buf, "%08lX%08lX", (ulong)enc->keyid[0], (ulong)enc->keyid[1]);
+	sprintf(buf, "%08lX%08lX %d 0",
+		(ulong)enc->keyid[0], (ulong)enc->keyid[1], enc->pubkey_algo );
 	write_status_text( STATUS_ENC_TO, buf );
     }
 
@@ -201,6 +231,8 @@ proc_pubkey_enc( CTX c, PACKET *pkt )
 		m_free(c->dek); c->dek = NULL;
 	    }
 	}
+	else
+	    result = G10ERR_NO_SECKEY;
     }
     else
 	result = G10ERR_PUBKEY_ALGO;
@@ -211,26 +243,77 @@ proc_pubkey_enc( CTX c, PACKET *pkt )
 	if( opt.verbose > 1 )
 	    log_info( _("public key encrypted data: good DEK\n") );
     }
-    else {
-	/* fixme: defer this message until we have parsed all packets of
-	 * this type - do this by building a list of keys with their stati
-	 * and store it with the context.  do_proc_packets can then use
-	 * this list to display some information */
-	log_error(_("public key decryption failed: %s\n"), g10_errstr(result));
+    else { /* store it for later display */
+	struct kidlist_item *x = m_alloc( sizeof *x );
+	x->kid[0] = enc->keyid[0];
+	x->kid[1] = enc->keyid[1];
+	x->pubkey_algo = enc->pubkey_algo;
+	x->reason = result;
+	x->next = c->failed_pkenc;
+	c->failed_pkenc = x;
     }
     free_packet(pkt);
 }
 
 
+
+/****************
+ * Print the list of public key encrypted packets which we could
+ * not decrypt.
+ */
+static void
+print_failed_pkenc( struct kidlist_item *list )
+{
+    for( ; list; list = list->next ) {
+	PKT_public_key *pk = m_alloc_clear( sizeof *pk );
+	const char *algstr = pubkey_algo_to_string( list->pubkey_algo );
+
+	pk->pubkey_algo = list->pubkey_algo;
+	if( !get_pubkey( pk, list->kid ) ) {
+	    size_t n;
+	    char *p;
+	    log_info( _("encrypted with %u-bit %s key, ID %08lX, created %s\n"),
+		       nbits_from_pk( pk ), algstr, (ulong)list->kid[1],
+		       strtimestamp(pk->timestamp) );
+	    fputs("      \"", log_stream() );
+	    p = get_user_id( list->kid, &n );
+	    print_string( log_stream(), p, n, '"' );
+	    m_free(p);
+	    fputs("\"\n", log_stream() );
+	}
+	else {
+	    log_info(_("encrypted with %s key, ID %08lX\n"),
+			algstr, (ulong) list->kid[1] );
+	}
+	free_public_key( pk );
+
+	if( list->reason == G10ERR_NO_SECKEY ) {
+	    log_info(_("no secret key for decryption available\n"));
+	    if( is_status_enabled() ) {
+		char buf[20];
+		sprintf(buf,"%08lX%08lX", (ulong)list->kid[0],
+					  (ulong)list->kid[1] );
+		write_status_text( STATUS_NO_SECKEY, buf );
+	    }
+	}
+	else
+	    log_error(_("public key decryption failed: %s\n"),
+						g10_errstr(list->reason));
+    }
+}
+
+
 static void
 proc_encrypted( CTX c, PACKET *pkt )
 {
     int result = 0;
 
+    print_failed_pkenc( c->failed_pkenc );
+
     /*log_debug("dat: %sencrypted data\n", c->dek?"":"conventional ");*/
     if( !c->dek && !c->last_was_session_key ) {
 	/* assume this is old conventional encrypted data */
-	c->dek = passphrase_to_dek( NULL,
+	c->dek = passphrase_to_dek( NULL, 0,
 		    opt.def_cipher_algo ? opt.def_cipher_algo
 					: DEFAULT_CIPHER_ALGO, NULL, 0 );
     }
@@ -255,8 +338,8 @@ proc_encrypted( CTX c, PACKET *pkt )
     else {
 	write_status( STATUS_DECRYPTION_FAILED );
 	log_error(_("decryption failed: %s\n"), g10_errstr(result));
-	/* FIXME: if this is secret key not available, try with
-	 * other keys */
+	/* Hmmm: does this work when we have encrypted using a multiple
+	 * ways to specify the session key (symmmetric and PK)*/
     }
     free_packet(pkt);
     c->last_was_session_key = 0;
@@ -440,7 +523,6 @@ do_check_sig( CTX c, KBNODE node, int *is_selfsig )
 }
 
 
-
 static void
 print_userid( PACKET *pkt )
 {
@@ -532,6 +614,7 @@ print_notation_data( PKT_signature *sig )
     /* TODO */
 }
 
+
 /****************
  * List the certificate in a user friendly way
  */
@@ -569,6 +652,13 @@ list_node( CTX c, KBNODE node )
 	    if( c->local_id )
 		putchar( get_ownertrust_info( c->local_id ) );
 	    putchar(':');
+	    if( node->next && node->next->pkt->pkttype == PKT_RING_TRUST) {
+		putchar('\n'); any=1;
+		if( opt.fingerprint )
+		    print_fingerprint( pk, NULL );
+		printf("rtv:1:%u:\n",
+			    node->next->pkt->pkt.ring_trust->trustval );
+	    }
 	}
 	else
 	    printf("%s  %4u%c/%08lX %s ",
@@ -603,6 +693,11 @@ list_node( CTX c, KBNODE node )
 		    putchar('\n');
 		    if( opt.fingerprint && !any )
 			print_fingerprint( pk, NULL );
+		    if( node->next
+			&& node->next->pkt->pkttype == PKT_RING_TRUST ) {
+			printf("rtv:2:%u:\n",
+				 node->next->pkt->pkt.ring_trust->trustval );
+		    }
 		    any=1;
 		}
 		else if( node->pkt->pkttype == PKT_PUBLIC_SUBKEY ) {
@@ -885,6 +980,7 @@ do_proc_packets( CTX c, IOBUF a )
 	      case PKT_PLAINTEXT:   proc_plaintext( c, pkt ); break;
 	      case PKT_COMPRESSED:  proc_compressed( c, pkt ); break;
 	      case PKT_ONEPASS_SIG: newpkt = add_onepass_sig( c, pkt ); break;
+	      case PKT_RING_TRUST:  newpkt = add_ring_trust( c, pkt ); break;
 	      default: newpkt = 0; break;
 	    }
 	}
@@ -962,8 +1058,8 @@ check_sig_and_print( CTX c, KBNODE node )
 			   : _("Good signature from \""));
 	    else
 		log_info(    _("                aka \""));
-	    print_string( log_stream(), un->pkt->pkt.user_id->name,
-					un->pkt->pkt.user_id->len, '\"' );
+	    print_utf8_string( log_stream(), un->pkt->pkt.user_id->name,
+					     un->pkt->pkt.user_id->len );
 	    fputs("\"\n", log_stream() );
 	    if( rc )
 		break; /* print only one id in this case */
@@ -1012,6 +1108,10 @@ check_sig_and_print( CTX c, KBNODE node )
 		     sig->pubkey_algo, sig->digest_algo,
 		     sig->sig_class, (ulong)sig->timestamp, rc );
 	write_status_text( STATUS_ERRSIG, buf );
+	if( rc == G10ERR_NO_PUBKEY ) {
+	    buf[16] = 0;
+	    write_status_text( STATUS_NO_PUBKEY, buf );
+	}
 	log_error(_("Can't check signature: %s\n"), g10_errstr(rc) );
     }
     return rc;
@@ -1058,8 +1158,8 @@ proc_tree( CTX c, KBNODE node )
 				     c->signed_data, c->sigfilename,
 			n1? (n1->pkt->pkt.onepass_sig->sig_class == 0x01):0 );
 	    else
-		rc = ask_for_detached_datafile( &c->mfx,
-					    iobuf_get_fname(c->iobuf));
+		rc = ask_for_detached_datafile( c->mfx.md, c->mfx.md2,
+					    iobuf_get_fname(c->iobuf), 0 );
 	    if( rc ) {
 		log_error("can't hash datafile: %s\n", g10_errstr(rc));
 		return;
@@ -1073,6 +1173,7 @@ proc_tree( CTX c, KBNODE node )
 	PKT_signature *sig = node->pkt->pkt.signature;
 
 	if( !c->have_data ) {
+	    /* detached signature */
 	    free_md_filter_context( &c->mfx );
 	    c->mfx.md = md_open(sig->digest_algo, 0);
 	    if( sig->digest_algo == DIGEST_ALGO_MD5
@@ -1080,15 +1181,16 @@ proc_tree( CTX c, KBNODE node )
 		/* enable a workaround for a pgp2 bug */
 		c->mfx.md2 = md_open( DIGEST_ALGO_MD5, 0 );
 	    }
-	  #if 0
-	    #warning md_start_debug enabled
-	    md_start_debug( c->mfx.md, "det1" );
-	    if( c->mfx.md2 )
-		md_start_debug( c->mfx.md2, "det2" );
-	  #endif
+	    else if( sig->digest_algo == DIGEST_ALGO_SHA1
+		     && sig->pubkey_algo == PUBKEY_ALGO_DSA
+		     && sig->sig_class == 0x01 ) {
+		/* enable the workaround also for pgp5 when the detached
+		 * signature has been created in textmode */
+		c->mfx.md2 = md_open( sig->digest_algo, 0 );
+	    }
 	    /* Here we have another hack to work around a pgp 2 bug
 	     * It works by not using the textmode for detached signatures;
-	     * this will let the first signazure check (on md) fail
+	     * this will let the first signature check (on md) fail
 	     * but the second one (on md2) which adds an extra CR should
 	     * then produce the "correct" hash.  This is very, very ugly
 	     * hack but it may help in some cases (and break others)
@@ -1096,10 +1198,11 @@ proc_tree( CTX c, KBNODE node )
 	    if( c->sigs_only )
 		rc = hash_datafiles( c->mfx.md, c->mfx.md2,
 				     c->signed_data, c->sigfilename,
-				  c->mfx.md2? 0 :(sig->sig_class == 0x01) );
+				     c->mfx.md2? 0 :(sig->sig_class == 0x01) );
 	    else
-		rc = ask_for_detached_datafile( &c->mfx,
-					    iobuf_get_fname(c->iobuf));
+		rc = ask_for_detached_datafile( c->mfx.md, c->mfx.md2,
+					 iobuf_get_fname(c->iobuf),
+			       c->mfx.md2? 0 :(sig->sig_class == 0x01) );
 	    if( rc ) {
 		log_error("can't hash datafile: %s\n", g10_errstr(rc));
 		return;
