@@ -65,15 +65,15 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
     }
     else if( !opt.use_embedded_filename ) {
 	fname = make_outfile_name( iobuf_get_real_fname(pt->buf) );
+	if( !fname )
+	    fname = ask_outfile_name( pt->name, pt->namelen );
 	if( !fname ) {
 	    rc = G10ERR_CREATE_FILE;
 	    goto leave;
 	}
     }
     else {
-	fname = m_alloc( pt->namelen +1 );
-	memcpy( fname, pt->name, pt->namelen );
-	fname[pt->namelen] = 0;
+	fname = make_printable_string( pt->name, pt->namelen, 0 );
     }
 
     if( nooutput )
@@ -166,10 +166,19 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	}
 	else { /* binary mode */
 	    byte *buffer = m_alloc( 32768 );
-	    for( ;; ) {
+	    int eof;
+	    for( eof=0; !eof; ) {
+		/* Why do we check for len < 32768:
+		 * If we won we would practically read 2 EOFS but
+		 * the first one has already popped the block_filter
+		 * off and therefore we don't catch the boundary.
+		 * Always assume EOF if iobuf_read returns less bytes
+		 * then requested */
 		int len = iobuf_read( pt->buf, buffer, 32768 );
 		if( len == -1 )
 		    break;
+		if( len < 32768 )
+		    eof = 1;
 		if( mfx->md )
 		    md_write( mfx->md, buffer, len );
 		if( fp ) {
@@ -243,18 +252,58 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
     return rc;
 }
 
+static void
+do_hash( MD_HANDLE md, MD_HANDLE md2, IOBUF fp, int textmode )
+{
+    text_filter_context_t tfx;
+    int c;
+
+    if( textmode ) {
+	memset( &tfx, 0, sizeof tfx);
+	iobuf_push_filter( fp, text_filter, &tfx );
+    }
+    if( md2 ) { /* work around a strange behaviour in pgp2 */
+	/* It seems that at least PGP5 converts a single CR to a CR,LF too */
+	int lc = -1;
+	while( (c = iobuf_get(fp)) != -1 ) {
+	    if( c == '\n' && lc == '\r' )
+		md_putc(md2, c);
+	    else if( c == '\n' ) {
+		md_putc(md2, '\r');
+		md_putc(md2, c);
+	    }
+	    else if( c != '\n' && lc == '\r' ) {
+		md_putc(md2, '\n');
+		md_putc(md2, c);
+	    }
+	    else
+		md_putc(md2, c);
+
+	    if( md )
+		md_putc(md, c );
+	    lc = c;
+	}
+    }
+    else {
+	while( (c = iobuf_get(fp)) != -1 ) {
+	    if( md )
+		md_putc(md, c );
+	}
+    }
+}
+
 
 /****************
  * Ask for the detached datafile and calculate the digest from it.
  * INFILE is the name of the input file.
  */
 int
-ask_for_detached_datafile( md_filter_context_t *mfx, const char *inname )
+ask_for_detached_datafile( MD_HANDLE md, MD_HANDLE md2,
+			   const char *inname, int textmode )
 {
     char *answer = NULL;
     IOBUF fp;
     int rc = 0;
-    int c;
 
     fp = open_sigfile( inname ); /* open default file */
     if( !fp && !opt.batch ) {
@@ -285,18 +334,12 @@ ask_for_detached_datafile( md_filter_context_t *mfx, const char *inname )
     if( !fp ) {
 	if( opt.verbose )
 	    log_info(_("reading stdin ...\n"));
-	while( (c = getchar()) != EOF ) {
-	    if( mfx->md )
-		md_putc(mfx->md, c );
-	}
-    }
-    else {
-	while( (c = iobuf_get(fp)) != -1 ) {
-	    if( mfx->md )
-		md_putc(mfx->md, c );
-	}
-	iobuf_close(fp);
+	fp = iobuf_open( NULL );
+	assert(fp);
     }
+    do_hash( md, md2, fp, textmode );
+    iobuf_close(fp);
+
 
   leave:
     m_free(answer);
@@ -304,30 +347,6 @@ ask_for_detached_datafile( md_filter_context_t *mfx, const char *inname )
 }
 
 
-static void
-do_hash( MD_HANDLE md, MD_HANDLE md2, IOBUF fp, int textmode )
-{
-    text_filter_context_t tfx;
-    int c;
-
-    if( textmode ) {
-	memset( &tfx, 0, sizeof tfx);
-	iobuf_push_filter( fp, text_filter, &tfx );
-    }
-    if( md2 ) { /* work around a strange behaviour in pgp2 */
-	while( (c = iobuf_get(fp)) != -1 ) {
-	    if( c == '\n' )
-	       md_putc(md2, '\r' );
-	    md_putc(md, c );
-	    md_putc(md2, c );
-	}
-    }
-    else {
-	while( (c = iobuf_get(fp)) != -1 )
-	    md_putc(md, c );
-    }
-}
-
 
 /****************
  * Hash the given files and append the hash to hash context md.
